
local exports = {}
exports.name = 'MonsterMaulersTool'
exports.version = '0.0.1'
exports.description = 'Monster Maulers Tool'
exports.author = { name = 'kalimag' }

local MonsterMaulersTool = exports

MonsterMaulersTool.damageTimeout = 4 -- duration in seconds that recent changes should be shown for
MonsterMaulersTool.textColor = 0xFFFFFFFF -- AARRGGBB
MonsterMaulersTool.backgroundColor = 0x88000000
MonsterMaulersTool.lineHeight = 10

function MonsterMaulersTool.startplugin()
	print('MonsterMaulersTool.startplugin()')

	MonsterMaulersTool.playerHealthHistory = {}
	MonsterMaulersTool.bossHealthHistory = {}

	local function getFadedColor(color, ratio)
		local alpha = math.floor((color >> 24) * ratio)
		return (color & 0x00FFFFFF) | (alpha << 24)
	end

	local function getFadedColors(age)
		local timeout = MonsterMaulersTool.damageTimeout

		local ratio = math.max(0, (timeout - age) / timeout)

		return getFadedColor(MonsterMaulersTool.textColor, ratio), getFadedColor(MonsterMaulersTool.backgroundColor, ratio)
	end

	local function appendHealth(history, health)
		local historyLength = #history
		if historyLength == 0 or health ~= history[historyLength] then
			history[historyLength + 1] = { health = health, clock = os.clock()}
		end
	end

	local function drawHealth(health, history, screen, x, y, scrollDirection)

		screen:draw_text(x, y, health, MonsterMaulersTool.textColor, MonsterMaulersTool.backgroundColor)

		if not history then
			return
		end

		local currentHealth = health

		for i = #history, 1, -1 do
			local prevChange = history[i]
			local diff = currentHealth - prevChange.health
			currentHealth = prevChange.health

			if diff ~= 0 then
				local age = os.clock() - prevChange.clock
				if age <= MonsterMaulersTool.damageTimeout then		
					y = y + (MonsterMaulersTool.lineHeight * (scrollDirection or 1))
					local fadedText, fadedBackground = getFadedColors(age)
					screen:draw_text(x, y, diff, fadedText, fadedBackground)				
				else
					break
				end
			end
		end
	end

	local function on_emu_frame_done()
		
		local rom = emu.romname()
		if rom ~= "mmaulers" and rom ~= "dadandrn" then 
			return
		end

		--print('MonsterMaulersTool.on_emu_frame_done()')
		
		local machine = manager:machine()
		local screen = machine.screens[":screen"]
		local ram = machine.devices[":maincpu"].spaces["program"]
	
		-- addresses from http://cheat.retrogames.com/mame_downloads.htm
		-- identical on both versions
		local playerHealth = ram:read_u16(0x400052)
		appendHealth(MonsterMaulersTool.playerHealthHistory, playerHealth)

		if playerHealth ~= 0 then
			drawHealth(playerHealth, MonsterMaulersTool.playerHealthHistory, screen, 45, 20)
		elseif #MonsterMaulersTool.playerHealthHistory > 0 then
			MonsterMaulersTool.playerHealthHistory = {}
		end
	
		local bossAddr = ram:read_u32(0x601EAC) -- i think?
		if bossAddr ~= 0 then
			local bossHealth = ram:read_u16(bossAddr + 0x52)			
			appendHealth(MonsterMaulersTool.bossHealthHistory, bossHealth)

			drawHealth(bossHealth, MonsterMaulersTool.bossHealthHistory, screen, 144, 208, -1)			
		elseif #MonsterMaulersTool.bossHealthHistory > 0 then
			MonsterMaulersTool.bossHealthHistory = {}
		end
	
	end

	emu.register_frame_done(on_emu_frame_done)
end

return exports
